@@ -41,11 +41,13 @@ class Dataset:
     # Default compression for n5 format
     n5_default_compressor = 'gzip' if AVAILABLE_COMPRESSORS['gzip'] else 'raw'
 
-    def __init__(self, dset_impl, handle, n_threads=1):
+    def __init__(self, dset_impl, handle, parent, name, n_threads=1):
         self._impl = dset_impl
         self._handle = handle
         self._attrs = AttributeManager(self._handle)
         self.n_threads = n_threads
+        self._parent = parent
+        self._name = name
 
     @staticmethod
     def _to_zarr_compression_options(compression, compression_options):
@@ -106,13 +108,15 @@ class Dataset:
     def _require_dataset(cls, group, name,
                          shape, dtype,
                          chunks, n_threads, **kwargs):
-        if group.has(name):
+        ghandle = group._handle
+        if ghandle.has(name):
 
-            if group.is_sub_group(name):
+            if ghandle.is_sub_group(name):
                 raise TypeError("Incompatible object (Group) already exists")
 
-            handle = group.get_dataset_handle(name)
-            ds = cls(_z5py.open_dataset(group, name), handle, n_threads)
+            handle = ghandle.get_dataset_handle(name)
+            ds = cls(_z5py.open_dataset(ghandle, name), handle, group,
+                     group._name + '/' + name, n_threads)
             if shape != ds.shape:
                 raise TypeError("Shapes do not match (existing (%s) vs new (%s))" % (', '.join(map(str, ds.shape)),
                                                                                      ', '.join(map(str, shape))))
@@ -144,6 +148,7 @@ class Dataset:
                         compression_options={}):
 
         # check shape, dtype and data
+        ghandle = group._handle
         have_data = data is not None
         if have_data:
             if shape is None:
@@ -175,7 +180,7 @@ class Dataset:
                                                                                 str(shape)))
         # limit chunks to shape
         chunks = tuple(min(ch, sh) for ch, sh in zip(chunks, shape))
-        is_zarr = group.is_zarr()
+        is_zarr = ghandle.is_zarr()
 
         # check compression / get default compression
         # if no compression is given
@@ -205,19 +210,20 @@ class Dataset:
         # convert the copts to json parseable string
         copts = json.dumps(copts)
         # get the dataset and write data if necessary
-        impl = _z5py.create_dataset(group, name, cls._dtype_dict[parsed_dtype],
-                                    shape, chunks, compression, copts)
-        handle = group.get_dataset_handle(name)
-        ds = cls(impl, handle, n_threads)
+        impl = _z5py.create_dataset(ghandle, name, cls._dtype_dict[parsed_dtype],
+                                    shape, chunks, compression, copts, fillvalue)
+        handle = ghandle.get_dataset_handle(name)
+        ds = cls(impl, handle, group, group._name + '/' + name, n_threads)
         if have_data:
             ds[:] = data
         return ds
 
     @classmethod
     def _open_dataset(cls, group, name):
-        ds = _z5py.open_dataset(group, name)
-        handle = group.get_dataset_handle(name)
-        return cls(ds, handle)
+        ghandle = group._handle
+        ds = _z5py.open_dataset(ghandle, name)
+        handle = ghandle.get_dataset_handle(name)
+        return cls(ds, handle, group, group._name + '/' + name)
 
     @property
     def is_zarr(self):
@@ -286,6 +292,14 @@ class Dataset:
         copts = json.loads(copts)
         return copts
 
+    @property
+    def parent(self):
+        return self._parent
+
+    @property
+    def name(self):
+        return self._name
+
     def __len__(self):
         return self._impl.len
 
@@ -466,17 +480,9 @@ class Dataset:
         Args:
             chunk_indices (tuple): indices of the chunk to write to
         Returns
-            np.ndarray
+            np.ndarray or None - chunk data, returns None if the chunk is empty
         """
-        chunk_reader = getattr(_z5py, 'read_chunk_%s' % self._impl.dtype)
-        # FIXME this is super hacky, but for some reason returning none in pybind raises
-        # an exception for integer types:
-        # "TypeError: int() argument must be a string,
-        # a bytes-like object or a number, not 'NoneType"
-        # so for now we catch the exception
-        # for float types, it doesn't, but returns an array with `nan`
-        try:
-            out = chunk_reader(self._impl, chunk_indices)
-        except TypeError:
+        if not self._impl.chunkExists(chunk_indices):
             return None
-        return None if np.isnan(out).any() else out
+        chunk_reader = getattr(_z5py, 'read_chunk_%s' % self._impl.dtype)
+        return chunk_reader(self._impl, chunk_indices)
